unit Digitizer_Main;

interface

{$mode delphi}{$H+}
uses 
  Graphics, LResources, Interfaces,
  SysUtils, Classes, Math, FileNameKeeper, iniFiles,
  Controls, ExtCtrls, Menus, ExtDlgs, Dialogs, StdCtrls, Spin, ComCtrls,
  Forms, lazjpg, Clipbrd,
{$ifdef Linux}
  unix, ContourMap;
{$else}
  Registry;
{$endif}

type

  { TMain }

  TMain = class(TForm)
    FileNameKeeper: TFileNameKeep;
    MainMenu1: TMainMenu;
    Help: TMenuItem;
    About: TMenuItem;
    Contents: TMenuItem;
    mmArea: TMenuItem;
    OpenPictureDialog: TOpenPictureDialog;
    Panel1: TPanel;
    Memo1: TMemo;
    Panel6: TPanel;
    //Panel7: TPanel;
    ScrollBox: TScrollBox;
    File1: TMenuItem;
    Open: TMenuItem;
    Save: TMenuItem;
    Exit1: TMenuItem;
    N1: TMenuItem;
    N2: TMenuItem;
    Shape1: TShape;
    Shape2: TShape;
    Shape3: TShape;
    Shape4: TShape;
    PopupMenu1: TPopupMenu;
    pmUndo: TMenuItem;
    N3: TMenuItem;
    ToolBar: TToolBar;
    tbOpen: TToolButton;
    ImageList: TImageList;
    tbSave: TToolButton;
    tbUndo: TToolButton;
    Edit: TMenuItem;
    Undo: TMenuItem;
    chkCSVformat: TCheckBox;
    SpinEdit: TSpinEdit;
    Panel2: TPanel;
    tbZoomIn: TToolButton;
    tbZoomOut: TToolButton;
    View: TMenuItem;
    ZoomIn: TMenuItem;
    ZoomOut: TMenuItem;
    Options: TMenuItem;
    mmMode: TMenuItem;
    PickingUp: TMenuItem;
    MeasureingLength: TMenuItem;
    mmX_log: TMenuItem;
    mmY_log: TMenuItem;
    SaveAsCSV: TMenuItem;
    N4: TMenuItem;
    N5: TMenuItem;
    NumColEdit: TSpinEdit;
    Image: TImage;
    Timer1: TTimer;
    SaveDialog: TSaveDialog;
    Panel4: TPanel;
    Timer2: TTimer;
    N7: TMenuItem;
    Z025: TMenuItem;
    Z050: TMenuItem;
    Z100: TMenuItem;
    Z200: TMenuItem;
    Z400: TMenuItem;
    ScanDigitize: TMenuItem;
    ImageAnalysis: TMenuItem;
    Tools: TMenuItem;
    GetColorStatistic: TMenuItem;
    GetNumPixel: TMenuItem;
    ProgressBar: TProgressBar;
    mmCumulative: TMenuItem;
    pmClear: TMenuItem;
    GetTotalLength: TMenuItem;
    procedure mmAreaClick(Sender: TObject);
    procedure OpenClick(Sender: TObject);
    procedure SaveClick(Sender: TObject);
    procedure AboutClick(Sender: TObject);
    procedure ContentsClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure CheckBox1Click(Sender: TObject);
    procedure CheckBox2Click(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure FileNameKeeperSelect(const SelectFileName: String);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure Shape1MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Shape2MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Shape3MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Shape4MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure pmUndoClick(Sender: TObject);
    procedure Memo1Enter(Sender: TObject);
    procedure Memo1Exit(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure tbZoomInClick(Sender: TObject);
    procedure tbZoomOutClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure chkCSVformatClick(Sender: TObject);
    procedure SaveAsCSVClick(Sender: TObject);
    procedure mmX_logClick(Sender: TObject);
    procedure mmY_logClick(Sender: TObject);
    procedure MeasureingLengthClick(Sender: TObject);
    procedure PickingUpClick(Sender: TObject);
    procedure ImageMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ImageMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure ImageMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormResize(Sender: TObject);
    procedure Timer2Timer(Sender: TObject);
    procedure ScanDigitizeClick(Sender: TObject);
    procedure ToolButton1Click(Sender: TObject);
    procedure Z025Click(Sender: TObject);
    procedure Z050Click(Sender: TObject);
    procedure Z100Click(Sender: TObject);
    procedure Z200Click(Sender: TObject);
    procedure Z400Click(Sender: TObject);
    procedure GetColorStatisticClick(Sender: TObject);
    procedure ImageAnalysisClick(Sender: TObject);
    procedure GetNumPixelClick(Sender: TObject);
    procedure mmCumulativeClick(Sender: TObject);
    procedure pmClearClick(Sender: TObject);
    procedure PopupMenu1Popup(Sender: TObject);
    procedure GetTotalLengthClick(Sender: TObject);
  private
    procedure ChangeMode;
    function GetReverseColor(Color: Integer): Integer;
    procedure LoadFigure(FileName: String);
    procedure Initiallization;
    procedure AxisConfiguration;
    procedure ReportConditions;
    procedure ShowSimpleMessage(S: String);
    procedure ChangeZoom(Size: Double);
    procedure GetColorInformation(x, y: Integer);
    procedure DecodeColor(Col: TColor; var R, G, B: Integer);
    function FileSaveQuery: String;
    function RoundToDecimalSF(Value : Extended; SigFig : Integer) : Extended;
    function SetLengthRJ(dumstr: String; dumint: Shortint): String;
    function Get_X_Value(X, Y: Integer): Double;
    function Get_Y_Value(X, Y: Integer): Double;
    function Brightness(Color: Integer): Double;
    function IsMeetTheCondition(x, y: Integer): Boolean;
    function WriteSign(x: Double): String;
  public
    bmp: TBitmap;
  end;

var
  Main: TMain;
  xs0, xs1, ys0, ys1, i, Xold, Yold, Xstart, Ystart: Integer;
  SigFig, Order, DS, Sign, ColCounter, Mode, Driver: ShortInt;
  x0, x1, y0, y1, Ax, Bx, Ay, By, Oxs, Oys, Ox, Oy, Kx, Ky, PicSize, V, asq, bsq, absq: Double;
  xs, ys: array[1..4] of Double;
  Vertex: array[1..3] of TPoint;
  Xsc, Ysc: array[1..2000] of Double;
  Msg: array[1..40] of String;
  x_, y_, CurrentFolder, dumstr, IniFileName: String;
  AllowClick, FirstScan, ChangeColor, BasedOnBrightness: Boolean;
  RedMin, RedMax, LANG: Integer;
  GRa1, GRa2, GRb1, GRb2, BRa1, BRa2, BRb1, BRb2, BrightnessLimit: Double;
  AlterColor: TColor;
  ExePath: String;
  {$ifdef Linux}
  f: TFont;
  AvFonts: TStringList;
  Locale: String;
  {$else}
  {$endif}

implementation

uses
  Simple_Message, About_Box, Help_Search, MyMsgDlg, Condition_Dialog;

procedure TMain.FormCreate(Sender: TObject);
var
  IniFile : TIniFile;
  {$ifdef MSWINDOWS}
  DeskTopFolder, MyDocDir: String;
  Registry : TRegistry;
  {$endif}
  DBool2: Boolean;
  S, ResourceFileName: String;
  i: Integer;
  procedure GetComboBoxItems(CB: TComboBox );
  begin
    i := CB.ItemIndex;
    CB.Items.Text := StringReplace(IniFile.ReadString(S, CB.Name +  '.Items.Text', ''), '<br>', '', [rfReplaceAll, rfIgnoreCase]);
    CB.ItemIndex := i;
  end;
begin
  ExePath := ExtractFilePath(Application.ExeName);
  Application.Title := 'SimpleDigitizer';
  FirstScan := True;
  AllowClick := True;
  OpenPictureDialog.Filter := 'Graphic file(*.jpg;*.JPG;*.jpeg;*.bmp;*.png;*.pnm)|*.jpg;*.JPG;*.jpeg;*.bmp;*.png;*.pnm';
  {$ifdef Linux}
  IniFileName := GetEnvironmentVariable('HOME') + '/.SimpleDigitizer.ini';
  {$else}
  {$endif}
  //----------------
  //DBool1 := True;
  //DBool2 := False;
  {$ifdef MSWINDOWS}
  Registry := TRegistry.Create;
  with Registry do
  try
    RootKey := HKEY_CURRENT_USER;
    OpenKey('Software\Microsoft\Windows\CurrentVersion\Explorer\Shell Folders', False);
    DeskTopFolder := ReadString('Desktop');
    //FontFolder := ReadString('Fonts');
    if FileExists(DeskTopFolder + '\SimpleDigitizer.LNK') = True then
      RenameFile(DeskTopFolder + '\SimpleDigitizer.LNK', DeskTopFolder + '\Simple Digitizer.LNK');
    MyDocDir := ReadString('Personal');// lpszPath; // Modified 02/01/05
    CloseKey;
  finally
    Free;
  end;
  S := ExtractFilePath(MyDocDir) + 'Application Data';
  if DirectoryExists(S) then
    IniFileName := S + '\SimpleDigitizer.ini'
  else
    IniFileName := ExePath + 'SimpleDigitizer.ini';
  {$else}
  {$endif}
  Msg[1] := 'At first, please click the leftmost point on the horizontal(x) axis where the value is known.';
  Msg[2] := 'Secondly, please click rightmost point on the x-axis where the value is known.';
  Msg[3] := 'Thirdly, please click bottommost point on the y-axis where the value is known.';
  Msg[4] := 'Fourthly, please click topmost point on the vertical(y) axis where the value is known.';
  Msg[5] := 'Please input the x-value.';
  Msg[6] := 'Please input the y-value.';
  Msg[7] := 'Is this logarithmic axis?';
  Msg[8] := 'Then, please click data you would like to digitize.';
  Msg[9] := 'Picked data have not been saved yet. Do you save them before closing?';
  Msg[10]:= 'File other than JPEG, PNG nor bitmap cannot be loaded.';
  Msg[11]:= 'No picture data in clipboard.';
  Msg[12]:= 'If an error message appears before scanning, please exit this software and retry.';
  Msg[13]:= 'Scanned image has not saved yet. Save it?';
  Msg[14]:= 'Please left-click at an edge of the segment whose length is known, and move toward another edge with your finger clicking, and release the finger on the edge.';
  Msg[15]:= 'Then, draw segment of which you want to measure the length.';
  Msg[16]:= 'What is the length? (without unit)';
  Msg[17]:= ' [Inverse Mode]';
  Msg[18]:= ' [Ruler Mode]';
  Msg[19]:= ' [Image Analysing Mode]';
  Msg[20]:= 'How many points do you want?';
  Msg[21]:= 'How many picsels do you sample?';
  Msg[22]:= 'Please follow the instructions for obtaining coodinate information.';
  Msg[23]:= 'Please specify the interval of scanning:';
  Msg[24]:= 'Do you know the width (cm) ?';
  S := Self.Name;
  ResourceFileName := ChangeFileExt(Application.ExeName, '.lng');
  {$ifdef Linux}
  Locale := GetEnvironmentVariable('LANG');
  {$endif}
  if FileExists(ResourceFileName) then
  begin
    IniFile := TIniFile.Create(ResourceFileName);
    with IniFile do
      try
        LANG                     := ReadInteger(S, 'LANG', 1);
        File1.Caption            := ReadString(S, 'File1.Caption', '');
        Open.Caption             := ReadString(S, 'Open.Caption', '');
        Save.Caption             := ReadString(S, 'Save.Caption', '');
        Exit1.Caption            := ReadString(S, 'Exit1.Caption', '');
        Edit.Caption             := ReadString(S, 'Edit.Caption', '');
        Undo.Caption             := ReadString(S, 'Undo.Caption', '');
        Tools.Caption            := ReadString(S, 'Tools.Caption', '');
        ScanDigitize.Caption     := ReadString(S, 'ScanDigitize.Caption', '');
        GetColorStatistic.Caption:= ReadString(S, 'GetColorStatistic.Caption', '');
        GetNumPixel.Caption      := ReadString(S, 'GetNumPixel.Caption', '');
        GetTotalLength.Caption   := ReadString(S, 'GetTotalLength.Caption', '');
        View.Caption             := ReadString(S, 'View.Caption', '');
        ZoomIn.Caption           := ReadString(S, 'ZoomIn.Caption', '');
        ZoomOut.Caption          := ReadString(S, 'ZoomOut.Caption', '');
        Options.Caption          := ReadString(S, 'Options.Caption', '');
        mmMode.Caption           := ReadString(S, 'mmMode.Caption', '');
        mmArea.Caption           := ReadString(S, 'mmArea.Caption', '');
        PickingUp.Caption        := ReadString(S, 'PickingUp.Caption', '');
        MeasureingLength.Caption := ReadString(S, 'MeasureingLength.Caption', '');
        ImageAnalysis.Caption    := ReadString(S, 'ImageAnalysis.Caption', '');
        mmX_log.Caption          := ReadString(S, 'mmX_log.Caption', '');
        mmY_log.Caption          := ReadString(S, 'mmY_log.Caption', '');
        SaveAsCSV.Caption        := ReadString(S, 'SaveAsCSV.Caption', '');
        mmCumulative.Caption       := ReadString(S, 'Cumulative.Caption', '');
        Help.Caption             := ReadString(S, 'Help.Caption', '');
        Contents.Caption         := ReadString(S, 'Contents.Caption', '');
        About.Caption            := ReadString(S, 'About.Caption', '');
        pmUndo.Caption           := ReadString(S, 'pmUndo.Caption', '');
        pmClear.Caption          := ReadString(S, 'pmClear.Caption', '');
        tbOpen.Hint              := ReadString(S, 'tbOpen.Hint', '');
        tbSave.Hint              := ReadString(S, 'tbSave.Hint', '');
        tbUndo.Hint              := ReadString(S, 'tbUndo.Hint', '');
        tbZoomIn.Hint            := ReadString(S, 'tbZoomIn.Hint', '');
        tbZoomOut.Hint           := ReadString(S, 'tbZoomOut.Hint', '');
        Panel4.Caption           := ReadString(S, 'Panel4.Caption', '');
        Panel6.Caption           := ReadString(S, 'Panel6.Caption', '');
        for i := 1 to 23 do
          Msg[i] := StringReplace(ReadString(S, 'Msg' + IntToStr(i), ''), '<br>', '', [rfReplaceAll, rfIgnoreCase]);
      finally
        Free;
      end;
  end;
  SpinEdit.Left := Panel4.Left + Panel4.Width + 10;
  IniFile := TIniFile.Create(IniFileName);
  with IniFile do
  try
    Mode   := ReadInteger('Settings', 'Mode', 1);
    DBool2 := ReadBool('Settings', 'csv', False);
    NumColEdit.Value := ReadInteger('Settings', 'Number of Colums', 1);
    Driver := ReadInteger('Settings', 'driver', 1);
    RedMin   := ReadInteger('Settings', 'RedMin', 0);
    RedMax   := ReadInteger('Settings', 'RedMax', 255);
    GRa1 := ReadFloat('Settings', 'GRa1', 0);
    GRa2 := ReadFloat('Settings', 'GRa2', 1000);
    GRb1 := ReadFloat('Settings', 'GRb1', 0);
    GRb2 := ReadFloat('Settings', 'GRb2', 256);
    BRa1 := ReadFloat('Settings', 'BRa1', 0);
    BRa2 := ReadFloat('Settings', 'BRa2', 1000);
    BRb1 := ReadFloat('Settings', 'BRb1', 0);
    BRb2 := ReadFloat('Settings', 'BRb2', 256);
    BrightnessLimit := ReadFloat('Settings', 'BrightnessLimit', 0.7);
    ChangeColor := ReadBool('Settings', 'ChangeColor', False);
    mmCumulative.Checked := ReadBool('Settings', 'Cumulative', False);
    mmArea.Checked := ReadBool('Settings', 'Area', False);
    AlterColor  := ReadInteger('Settings', 'AlterColor', clRed);
    CurrentFolder := ReadString('Settings', 'CurrentFolder', ExePath);
   // Font.Name     := ReadString('Settings', 'Font.Name', Font.Name);
    BasedOnBrightness := ReadBool('Settings', 'BasedOnBrightness', False);
  finally
    IniFile.Free;
  end;

  case Mode of
  1: PickingUpClick(Sender);
  2: MeasureingLengthClick(Sender);
  3: ImageAnalysisClick(Sender);
  end;
  if DBool2 = True then
    SaveAsCSVClick(Sender);
  IniFile := TIniFile.Create(IniFileName);
  try
    with IniFile do
    begin
      S := ExtractFilePath(ReadString('KeepFileNames', 'FileName1', ''));
      S := StringReplace(S, '..', FileNameKeeper.MyDocDir, []);
      OpenPictureDialog.InitialDir := S;
    end;
  finally
    IniFile.Free;
  end;
  if OpenPictureDialog.InitialDir = '' then
    OpenPictureDialog.InitialDir := ExePath;
  //Paste.Visible   := False;
  //pmPaste.Visible := False;


  x0 := 0;
  x1 := 1;
  y0 := 0;
  y1 := 1;
  {
  if ParamCount > 0 then
  begin
    str := ParamStr(1);
    for i := 2 to Paramcount do
      str := str + ' ' + ParamStr(i);
    LoadFigure(str);
  end;
  }
  bmp := TBitmap.Create;
  //Map.TargetCanvas := Image.Canvas;
end;

procedure TMain.FormShow(Sender: TObject);
var
  SearchRec: TSearchRec;
  i: Integer;
  S, S2: String;
begin
  ToolBar.Visible := True;
  if ParamCount > 0 then
  begin
    S := ParamStr(1);
    for i := 2 to ParamCount do
      S := S + ' ' + ParamStr(i);
    FindFirst(S, faAnyFile, SearchRec);
    if Copy(S, Length(ExtractFileDir(S)),1) = '\' then S2 := '' else S2 := '\';
    {$ifdef Linux}
    S := ExtractFileDir(S) + S2 + SearchRec.Name;
    {$else}
    {$Warnings off}
    S := ExtractFileDir(S) + S2 + SearchRec.FindData.cFileName;
    {$Warnings on}
    {$endif}
    SysUtils.FindClose(SearchRec);
    S2 := ExtractFileExt(S);
    if (S2 = '.jpg') or (S2 = '.bmp') or (S2 = '.png') then
    begin
      AboutClick(Sender);
      LoadFigure(S);
    end;
  end;

  //WindowState := wsMaximized;
  {$ifdef Linux}
  Left := 0;
  Width  := Screen.Width;
  Height := Screen.Height;
  {$endif}
  //Application.OnHelp := HelpSearch.HelpTreeView.AppHelp;
  Top := -10;
  //Memo1.Left := Width - Memo1.Width - 20;
  ScrollBox.Height := Screen.Height - ScrollBox.Top - 100;
  Memo1.Height  := ScrollBox.Height;
  Panel1.Height := ScrollBox.Height - 4;
  ScrollBox.Width := Screen.Width - ScrollBox.Left - Memo1.Width - 30;
  Panel1.Width  := ScrollBox.Width - 8;
  Options.Click;
  Caption := 'SimpleDigitizer - ' + Msg[Mode + 16];
end;

procedure TMain.OpenClick(Sender: TObject);
begin
  if FileSaveQuery = 'Cancel' then
    SysUtils.Abort;
  with OpenPictureDialog do
  begin
    InitialDir := CurrentFolder;
    if Execute = True then
      if FilterIndex = 1 then
        LoadFigure(FileName);
  end;
end;

procedure TMain.ImageMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var dum: Integer;
begin
  if (MeasureingLength.Checked) and (((mmCumulative.Checked = False) and (mmArea.Checked = False)) or (((mmCumulative.Checked) or (mmArea.Checked)) and (xs[3] = 0))) then
  begin
    if Button = mbRight then
      Exit;
    Xstart := x;
    Ystart := y;
    dum := Image.Picture.Bitmap.Height;
    x := Trunc(x / PicSize);
    y := Trunc((dum - y) / PicSize);
    if Order = 1 then
    begin
      xs[1] := x;
      ys[1] := y;
    end
    else
    begin
      xs[3] := x;
      ys[3] := y;
    //  i := i + 1;
    //  Xsc[i] := (PicSize * x) / dum;
    //  Ysc[i] := Trunc(dum - PicSize * y) / dum;
    end;
  end;
end;

procedure TMain.ImageMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
var
 // CP: TPoint;
  i: Integer;
begin
  if (Mode = 1) and (Order = 0) and (xs[1] > 0) then
  begin
    x := Trunc(x / PicSize);
    y := Trunc((Image.Picture.Bitmap.Height - y) / PicSize);
    if x0 > 0 then
      SpinEdit.Value := Max(2 + Trunc(log10(x0)), SpinEdit.Value);
    SigFig := SpinEdit.Value;
    x_ := FloatToStr(Get_X_Value(x, y));
    y_ := FloatToStr(Get_Y_Value(x, y));
   // Panel7.Caption := x_ + ' ,  ' + y_;
  end
  else
    if ((MeasureingLength.Checked) and (Shift = [ssLeft]))
     or ((Mode = 1) and ((Order = 2) or (Order = 4))) then //or ((mmCumulative.Checked) and (xs[3] > 0))) then
      with Image.Canvas do
      begin
      //  GetCursorPos(CP);
        if (x = Xold) and (y = Yold) then
          Exit;
        {$ifdef Linux}
        Pen.Mode := pmNotXor;
        {$else}
        Pen.Mode := pmXor;
        {$endif}
        Moveto(Xstart, Ystart);
        if (Xold > 0) and (Yold > 0) then
          Lineto(Xold, Yold);
        Xold := x; Yold := y;
        Moveto(Xstart, Ystart);
        Lineto(x, y);
       // SetCursorPos(CP.x, Cp.y);
        {$ifdef Linux}
        i := 3;
        {$else}
        i := 1;
        {$endif}
        if (X - ScrollBox.HorzScrollBar.Position) > ScrollBox.Width - 40 then
        begin
          ScrollBox.HorzScrollBar.Position := ScrollBox.HorzScrollBar.Position + i;
          Application.ProcessMessages;
        end;
        if (X - ScrollBox.HorzScrollBar.Position) < 40 then
        begin
          ScrollBox.HorzScrollBar.Position := ScrollBox.HorzScrollBar.Position - i;
          Application.ProcessMessages;
        end;
        if (Y - ScrollBox.VertScrollBar.Position) > ScrollBox.Height - 40 then
        begin
          ScrollBox.VertScrollBar.Position := ScrollBox.VertScrollBar.Position + i;
          Application.ProcessMessages;
        end;
        if (Y - ScrollBox.VertScrollBar.Position) < 40 then
        begin
          ScrollBox.VertScrollBar.Position := ScrollBox.VertScrollBar.Position - i;
          Application.ProcessMessages;
        end;
      end;
end;

procedure TMain.ImageMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  d : Double;
  dum, yd1, yd2: Integer;
  xo, yo: Integer;
  a, b: TPoint;
begin
  if Button = mbRight then
    Exit;
 // if Order <> 0 then Image.Canvas.Rectangle(x-3, y-3, x+3, y+3);
  dum := Image.Picture.Bitmap.Height;
  xo := x;
  yo := y;
  x := Trunc(x / PicSize);
  y := Trunc((dum - y) / PicSize);
  xold := 0;
  yold := 0;
  Image.Canvas.Pen.Color := clLime;
  case Mode of
  1:begin
      {$ifdef Linux}

      {$else}
      //sndPlaySound('Hit1.wav', SND_SYNC);
      {$endif}
      if Order = 1 then
      begin
        Shape1.Left := Trunc(x * PicSize) + 1;
        Shape1.Top  := Trunc(dum - y * PicSize) + 1;
        Shape1.Visible := True;
        //xs0 := x;
        xs[1] := x;
        ys[1] := y;
        Xstart := xo;
        Ystart := yo;
        SimpleMessage.Close;
        x0  := MyMessageDlg.ReceiveFloat('SimpleDigitizer', Msg[5], ' ', x0, 40, 4);
        Application.ProcessMessages;
        ShowSimpleMessage(Msg[2]);
        Order := 2;
        Undo.Enabled := True;
        tbUndo.Enabled := True;
        pmUndo.Enabled := True;
        Exit;
      end;
      if Order = 2 then
      begin
        Shape2.Left := Trunc(x * PicSize) + 1;
        Shape2.Top  := Trunc(dum - y * PicSize) + 1;
        Shape2.Visible := True;
       // xs1:= x;
        xs[2] := x;
        ys[2] := y;
        SimpleMessage.Close;
        x1  := MyMessageDlg.ReceiveFloat('SimpleDigitizer', Msg[5], ' ', x1, 40, 4);
        if mmX_log.Checked = False then
          if (x0 > 0) and (x1 > 0) then
          begin
            d := Abs(Log10(x0/x1));
            if (d > 1.9999) or (d - Trunc(d) = 0) then
              if MessageDlg(Msg[7], mtConfirmation, [mbYes, mbNo], 0) = mrYes then
           //   if Application.MessageBox(PChar(Msg[7]), '', mb_YesNo + mb_IconQuestion) = idYes then
              begin
                mmX_log.Checked := True;
                mmX_log.Enabled := False;
              end;
          end;
        //dumint := Abs(Ceil(log10(Abs((x1 - x0) / 1000))));
        //if dumint > 0 then
        //  SpinEdit.Text := IntToStr(dumint);
        ShowSimpleMessage(Msg[3]);
        Order := 3;
        Exit;
      end;
      if Order = 3 then
      begin
        Shape3.Left := Trunc(x * PicSize) + 1;
        Shape3.Top  := Trunc(dum - y * PicSize) + 1;
        Shape3.Visible := True;
       // ys1:= y;
        xs[4] := x;
        ys[4] := y;
        Xstart := xo;
        Ystart := yo;
        SimpleMessage.Close;
        y0  := MyMessageDlg.ReceiveFloat('SimpleDigitizer', Msg[6], ' ', y0, 40, 4);
        ShowSimpleMessage(Msg[4]);
        Order := 4;
        Exit;
      end;
      if Order = 4 then
      begin
        Order := 0;
        Shape4.Left := Trunc(x * PicSize) + 1;
        Shape4.Top  := Trunc(dum - y * PicSize) + 1;
        Shape4.Visible := True;
       // ys0:= y;
        xs[3] := x;
        ys[3] := y;
        SimpleMessage.Close;
        y1  := MyMessageDlg.ReceiveFloat('SimpleDigitizer', Msg[6], ' ', y1, 40, 4);
        if mmY_log.Checked = False then
          if (y0 > 0) and (y1 > 0) then
          begin
            d := Abs(Log10(y0/y1));
            if (d > 1.9999) or (d - Trunc(d) = 0) then
              if MessageDlg(Msg[7], mtConfirmation, [mbYes, mbNo], 0) = mrYes then
              //  if Application.MessageBox(PChar(Msg[7]), '', mb_YesNo + mb_IconQuestion) = idYes then
                begin
                  mmY_log.Checked := True;
                  mmY_log.Enabled := False;
                end;
          end;
        //dumint := Abs(Ceil(log10(Abs((y1 - y0) / 1000))));
        //if dumint > 0 then
        //  Edit2.Text := IntToStr(dumint);
        ShowMessage(Msg[8]);
        if (mmX_log.Checked) and (x0 > 0) and (x1 > 0) then
        begin
          x0 := Log10(x0);
          x1 := Log10(x1);
        end
        else
          mmX_log.Checked := False;
        if (mmY_log.Checked) and (y0 > 0) and (y1 > 0) then
        begin
          y0 := Log10(y0);
          y1 := Log10(y1);
        end
        else
          mmY_log.Checked := False;
        AxisConfiguration;
        Save.Enabled   := True;
        tbSave.Enabled := True;
        ScanDigitize.Enabled := True;
        Exit;
      end;
      if (Order = 0) and (AllowClick = True) then
      begin
        Undo.Enabled := False;
        tbUndo.Enabled := False;
        pmUndo.Enabled := False;
        i := i + 1;
        SigFig := SpinEdit.Value;
        yd1 := Trunc(dum - PicSize * y - 2);
        yd2 := yd1 + 6;
        Image.Canvas.Brush.Style := bsClear;// .Col;or := clLime;
        Image.Canvas.Ellipse(Trunc(PicSize * x) - 2, yd1, Trunc(PicSize * x) + 4, yd2);
        Xsc[i] := (PicSize * x) / dum;
        Ysc[i] := (yd1 + 2) / dum;
        x_ := FloatToStr(Get_X_Value(x, y));
        y_ := FloatToStr(Get_Y_Value(x, y));
        if chkCSVformat.Checked then
          dumstr := ','
        else
          dumstr := ' ';
        Memo1.Lines.Add(SetLengthRJ(x_, 10) + dumstr + SetLengthRJ(y_, 10));
      end;
    end;
  2:begin
      if Order = 1 then
      begin
        //xs0 := x;
        xs[2] := x;
        ys[2] := y;
        SimpleMessage.Close;
        x0  := MyMessageDlg.ReceiveFloat('SimpleDigitizer', Msg[16], ' ', 1, 40, 4);
        Kx := x0 / Sqrt(Sqr(Xs[1] - Xs[2]) + Sqr(Ys[1] - Ys[2]));
        Application.ProcessMessages;
        ShowSimpleMessage(Msg[15]);
        Order := 0;
        Undo.Enabled := True;
        tbUndo.Enabled := True;
        pmUndo.Enabled := True;
        ColCounter := 0;
        with Image.Canvas do
        begin
          Pen.Mode  := pmNot;
          Moveto(Xstart, Ystart);
          Lineto(xo, yo);
        end;
      end
      else
      begin
        with Image.Canvas do
        begin
          Pen.Color := clBlue;
          Pen.Mode  := pmCopy;
          Moveto(Xstart, Ystart);
          Lineto(xo, yo);
          Xstart := xo;
          Ystart := yo;
        end;
        SigFig := SpinEdit.Value;
        i := i + 1;
        Xsc[i] := (PicSize * x) / dum;
        Ysc[i] := Trunc(dum - PicSize * y) / dum;
       // Memo1.Lines.Add(IntToStr(xo) + ', ' + IntToStr(yo));
        if mmArea.Checked then
        begin
          {
          Map.X[i] := xo;
          Map.Y[i] := yo;
          }
          if Vertex[1].X > 0 then
          begin
            if Vertex[2].X > 0 then
            begin
              if Vertex[3].X > 0 then
                Vertex[2]  := Vertex[3];
              Vertex[3].X := xo;
              Vertex[3].Y := yo;
              Image.Canvas.Brush.Style:= bsDiagCross;
              Image.Canvas.Polygon(Vertex);
              a.X := Vertex[2].X - Vertex[1].X ;
              b.X := Vertex[3].X - Vertex[1].X ;
              a.Y := Vertex[2].Y - Vertex[1].Y ;
              b.Y := Vertex[3].Y - Vertex[1].Y ;
              asq := a.X * a.X + a.Y * a.Y;
              bsq := b.X * b.X + b.Y * b.Y;
              absq:= a.X * b.X + a.Y * b.Y;
              absq:= Sqr(absq);
              V := V + 0.5 * Sqrt(asq * bsq - absq) * Sqr(kx);
              Memo1.Lines.Add(SetLengthRJ(FloatToStr(RoundToDecimalSF(V, SigFig)), 10));
            end
            else
            begin
              Vertex[2].X := xo;
              Vertex[2].Y := yo;
            end;
          end
          else
          begin
            Vertex[1].X := xo;
            Vertex[1].Y := yo;
          end;
        end
        else
        begin
          if not mmCumulative.Checked then
            V := 0;
          V := V + Kx * Sqrt(Sqr(x - Xs[3]) + Sqr(y - Ys[3]));
          xs[3] := x;
          ys[3] := y;
          ColCounter := ColCounter + 1;
          if (ColCounter > 1) and (ColCounter < NumColEdit.Value + 1) then
          begin
            if chkCSVformat.Checked then
              dumstr := ', ' else dumstr := '  ';
            Memo1.Lines[Memo1.Lines.Count-1] := Memo1.Lines[Memo1.Lines.Count-1] + dumstr + SetLengthRJ(FloatToStr(RoundToDecimalSF(V, SigFig)), 10)
          end
          else
          begin
            Memo1.Lines.Add(SetLengthRJ(FloatToStr(RoundToDecimalSF(V, SigFig)), 10));
            ColCounter := 1;
          end;
        end;
      end;
    end;
  3:begin
      GetColorInformation(Trunc(PicSize * x), Trunc(Image.Picture.Bitmap.Height - PicSize * y));
    end;
  end;
end;

function TMain.Get_X_Value(X, Y: Integer): Double;
var l: Double;
begin
  if Ay > 0 then
    l := Sign * (- Ay * x + y - By) / Sqrt(Sqr(Ay) + 1)   // distance to Y-coodinate
  else
    l := x - Oxs;
  if mmX_log.Checked then
    Result := RoundToDecimalSF(Power(10, Ox + l * Kx), SigFig)
  else
    Result := RoundToDecimalSF(Ox + l * Kx, SigFig);
end;

function TMain.Get_Y_Value(X, Y: Integer): Double;
var l: Double;
begin
  l := (- Ax * x + y - Bx) / Sqrt(Sqr(Ax) + 1);   // distance to X-coodinate
  if mmY_log.Checked then
    Result := RoundToDecimalSF(Power(10, Oy + l * Ky), SigFig)
  else
    Result := RoundToDecimalSF(Oy + l * Ky, SigFig);
end;

procedure TMain.AxisConfiguration;
begin
  Ax := (ys[2] - ys[1]) / (xs[2] - xs[1]);
  Bx := ys[1] - Ax * xs[1];
  if Ax <> 0 then
  begin
    Ay := - 1 / Ax; //(ys[3] - ys[4]) / (xs[3] - xs[4]);
    By := ys[3] - Ay * xs[3];
    Oxs := (By - Bx) / (Ax - Ay);
  end
  else
    Oxs := xs[3];
  if Ax >= 0 then Sign := 1 else Sign := -1;
  Oys := Ax * Oxs + Bx;
  Ox := x1 - (xs[2] - Oxs) * (x1 - x0) / (xs[2] - xs[1]) ;
  Oy := y1 - (ys[3] - Oys) * (y1 - y0) / (ys[3] - ys[4]) ;
  Kx := (x1 - x0) / Sqrt(Sqr(xs[2] - xs[1]) + Sqr(ys[2] - ys[1]));
  Ky := (y1 - y0) / Sqrt(Sqr(xs[4] - xs[3]) + Sqr(ys[4] - ys[3]));
end;

procedure TMain.SaveClick(Sender: TObject);
begin
  AllowClick := False;
  DS := DS + 1;
  with SaveDialog do
  begin
    if FilterIndex = 1 then
    begin
      if chkCSVformat.Checked then
        dumstr := '.csv'
      else
        dumstr := '.txt';
      FileName := ChangeFileExt(Copy(Main.Caption, 19, 100), '') + '_' + IntToStr(DS) + dumstr;
    end
    else
      FileName := OpenPictureDialog.FileName;
    if Execute then
    begin
      if (ExtractFileExt(FileName) = '.txt') or (ExtractFileExt(FileName) = '.csv') then
      begin
        Memo1.Lines.SaveToFile(FileName);
        Memo1.Lines.Clear;
      end
      else
      begin
        if ExtractFileExt(FileName) = '.png' then
          FileName := ChangeFileExt(FileName, '.bmp');
        Image.Picture.SaveToFile(FileName);
        FileNameKeeper.SetFileName(FileName);
        SaveDialog.FilterIndex := 0;
      end;
    end;
  end;
  AllowClick := True;
end;

function TMain.SetLengthRJ(dumstr: String; dumint: Shortint): String;
var i: Shortint;
begin
  for i:=1 to dumint - Length(dumstr) do
    dumstr:=' '+ dumstr;
  Result := dumstr;
end;
//----------------------------------------------------------
procedure TMain.AboutClick(Sender: TObject);
begin
  with AboutBox do
  begin
    ProductName.Caption := 'SimpleDigitizer';
    {$ifdef Linux}
    Version.Caption := 'Version 3.2';
    ProgramIcon.Picture.LoadFromFile(ExePath + 'SimpleDigitizer.png');
    {$else}
    Version.Caption := 'Version 3.2';
    ProgramIcon.Picture.LoadFromFile(ExePath + 'Digitizer.ico');
    {$endif}
    Show;
  end;
end;

procedure TMain.ContentsClick(Sender: TObject);
begin
  HelpSearch.Show;
end;

procedure TMain.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
(*
{$ifdef Linux}
  if Key = 4000 then
{$else}
  if Key = VK_F1 then
{$endif}
    ContentsClick(Self);
*)
end;

procedure TMain.CheckBox1Click(Sender: TObject);
begin
  {
  with mmmmX_log.Checked do
    if Checked  then
    begin
      Checked := False ;
      mmX_log.Checked := True;
    end
    else
    begin
      Checked := True;
      mmX_log.Checked := False;;
    end
   }
end;

procedure TMain.CheckBox2Click(Sender: TObject);
begin
  {
  with mmmmY_log.Checked do
    if Checked  then
    begin
      Checked := False ;
      mmY_log.Checked := True;
    end
    else
    begin
      Checked := True;
      mmY_log.Checked := False;;
    end
   }
end;

procedure TMain.Exit1Click(Sender: TObject);
begin
  Close;
end;

procedure TMain.FileNameKeeperSelect(const SelectFileName: String);
begin
  if FileSaveQuery = 'Cancel' then
    SysUtils.Abort;
  LoadFigure(SelectFileName);
end;

procedure TMain.LoadFigure(FileName: String);
var
  jpg: TJPGImage;
  png: TPortableNetworkGraphic;
  S, S1: String;
begin
  S := AnsiLowerCase(ExtractFileExt(FileName));
  Image.Enabled := True;
  ScanDigitize.Enabled := False;
  Caption := 'SimpleDigitizer - ' + ExtractFileName(FileName) + ' ' + Msg[Mode + 16];
  CurrentFolder := ExtractFileDir(FileName);
  FileNameKeeper.SetFileName(FileName);
  S1 := AnsiLowerCase(ExtractFileExt(FileName));
  //bmp.LoadFromFile(FileName);
  //Image.Picture.Bitmap.Assign(bmp);
  if (S1 = '.jpg') or (S1 = '.jpeg') then
  begin
    jpg := TJPGImage.Create;
    try
      jpg.LoadFromFile(FileName);
      bmp.Assign(jpg);
      Image.Picture.Bitmap.Assign(jpg);
    finally
      jpg.free;
    end;
  end
  else
    if S1 = '.png' then
    begin
      png := TPortableNetworkGraphic.Create;
      try
        png.LoadFromFile(FileName);
        bmp.Assign(png);
        Image.Picture.Bitmap.Assign(png);
      finally
        png.free;
      end;
    end
    else
      if S1 = '.bmp' then
      begin
        bmp.LoadFromFile(FileName);
        Image.Picture.Bitmap.Assign(bmp);
      end
      else
      begin
        ShowSimpleMessage(Msg[10]);
        Exit;
      end;
  mmMode.Enabled := False;
  Initiallization;
  {$ifdef Linux}
  Timer1Timer(Self);
  {$endif}
end;

procedure TMain.Initiallization;
var j: Integer;
begin
  Shape1.Visible := False;
  Shape2.Visible := False;
  Shape3.Visible := False;
  Shape4.Visible := False;
  Memo1.Lines.Clear;
  mmX_log.Checked := False;
  mmY_log.Checked := False;
  mmX_log.Checked := False;
  mmY_log.Checked := False;
  if Mode = 1 then
  begin
    mmX_log.Enabled := True;
    mmY_log.Enabled := True;
  end;
  mmY_log.Checked := False;
  Image.Cursor := crDefault;
  DS := 0;
  if Image.Picture.Width > ScrollBox.Width then
    Panel1.Width  := Image.Picture.Width;
  if Image.Picture.Height > ScrollBox.Height then
    Panel1.height := Image.Picture.height;
  i := 0;
  if Mode <= 2 then
    ShowMessage(Msg[22]);
  Order := 1;
  case Mode of
  1: ShowSimpleMessage(Msg[1]);
  2:begin
      if x0 = 0 then
        x0 := 1;
      if MessageDlg(Msg[24], mtConfirmation, [mbYes, mbNo], 0) = mrYes then
      begin
        x0  := MyMessageDlg.ReceiveFloat('SimpleDigitizer', Msg[16], ' ', x0, 40, 4);
        Kx := x0 / bmp.Width;
        Application.ProcessMessages;
        ShowSimpleMessage(Msg[15]);
        Order := 0;
        Undo.Enabled := True;
        tbUndo.Enabled := True;
        pmUndo.Enabled := True;
        ColCounter := 0;
      end
      else
        ShowSimpleMessage(Msg[14]);
    end;
  3: Memo1.Lines.Add('red, green, blue');
  end;
  Image.Enabled := True;
  Image.Canvas.Pen.Color := clBlue;
  if PickingUp.Checked = True then
  begin
    Undo.Enabled := True;
    tbUndo.Enabled := True;
    pmUndo.Enabled := True;
  end;
  //Save.Enabled   := False;
  Image.Cursor := crCross;
  PicSize := 1;
  for j := 1 to 2000 do
  begin
    Xsc[j] := 0;
    Ysc[j] := 0;
  end;
  pmClearClick(Self);
  tbSave.Enabled := Save.Enabled;
end;

function TMain.RoundToDecimalSF(Value : Extended; SigFig : Integer) : Extended;
var
  Places: ShortInt;
  N : Extended;
begin
  if Value <> 0 then
  begin
    Places := SigFig - Ceil(log10(Abs(Value)));
    N := Power(10.0, Places);
    RoundToDecimalSF := (Round(Value * N) )/ N;
  end
  else
    RoundToDecimalSF := Value;
end;

procedure TMain.FormClose(Sender: TObject; var Action: TCloseAction);
var IniFile: TIniFile;
begin
  if FileSaveQuery = 'Cancel' then
    SysUtils.Abort;
  bmp.Free;
  IniFile := TIniFile.Create(IniFileName);
  try
    with IniFile do
    begin
      WriteString('Settings', 'CurrentFolder', CurrentFolder);
      WriteString('Settings', 'Font.Name', Font.Name);
      WriteInteger('Settings', 'Mode', Mode);
      WriteInteger('Settings', 'Driver', Driver);
      WriteInteger('Settings', 'RedMin', RedMin);
      WriteInteger('Settings', 'RedMax', RedMax);
      WriteFloat('Settings', 'GRa1', GRa1);
      WriteFloat('Settings', 'GRa2', GRa2);
      WriteFloat('Settings', 'GRb1', GRb1);
      WriteFloat('Settings', 'GRb2', GRb2);
      WriteFloat('Settings', 'BRa1', BRa1);
      WriteFloat('Settings', 'BRa2', BRa2);
      WriteFloat('Settings', 'BRb1', BRb1);
      WriteFloat('Settings', 'BRb2', BRb2);
      WriteFloat('Settings', 'BrightnessLimit', BrightnessLimit);
      WriteBool('Settings', 'ChangeColor', ChangeColor);
      WriteBool('Settings', 'Cumulative', mmCumulative.Checked);
      WriteBool('Settings', 'Area', mmArea.Checked);
      WriteBool('Settings', 'BasedOnBrightness', BasedOnBrightness);
      WriteInteger('Settings', 'AlterColor', AlterColor);
      if ImageAnalysis.Checked = True then
      begin
        WriteInteger('Settings', 'Number of Colums', NumColEdit.Value);
        WriteBool('Settings', 'csv', SaveAsCSV.Checked);
      end;
      UpdateFile;
    end;
  finally
    IniFile.Free;
  end;
end;

function TMain.FileSaveQuery: String;
begin
  Result := '';
  if Memo1.Lines.Count > 1 then
    Case MessageDlg(Msg[9], mtConfirmation, [mbYes, mbNo, mbCancel], 0) of
  //  Case Application.MessageBox(PChar(Msg[9]), '', mb_YesNoCancel + mb_IconQuestion) of
      mrYes: SaveClick(Main);
      mrCancel: Result := 'Cancel';
    end;
end;

procedure TMain.Shape1MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  ImageMouseUp(Sender, mbLeft, [], Shape1.Left - Panel1.BevelWidth + x, Shape1.Top - Panel1.BevelWidth + y);
end;

procedure TMain.Shape2MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  ImageMouseUp(Sender, mbLeft, [], Shape2.Left - Panel1.BevelWidth + x, Shape2.Top - Panel1.BevelWidth + y);
end;

procedure TMain.Shape3MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  ImageMouseUp(Sender, mbLeft, [], Shape3.Left - Panel1.BevelWidth + x, Shape3.Top - Panel1.BevelWidth + y);
end;

procedure TMain.Shape4MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  ImageMouseUp(Sender, mbLeft, [], Shape4.Left - Panel1.BevelWidth + x, Shape4.Top - Panel1.BevelWidth + y);
end;

procedure TMain.pmUndoClick(Sender: TObject);
begin
  if Order > 1 then
    Order := Order - 1;
  if Order = 0 then
    Order := 4;
  Case Order of
  1:begin
      Shape1.Visible := False;;
      ShowSimpleMessage(Msg[1]);
    end;
  2:begin
      Shape2.Visible := False;;
      mmX_log.Enabled := True;
      ShowSimpleMessage(Msg[2]);
    end;
  3:begin
      Shape3.Visible := False;;
      ShowSimpleMessage(Msg[3]);
    end;
  4:begin
      Image.Cursor := crDefault;
      Shape4.Visible := False;;
      mmY_log.Enabled := True;
      ShowSimpleMessage(Msg[4]);
    end;
  end;
end;


procedure TMain.Memo1Enter(Sender: TObject);
begin
  Main.PopupMenu := nil;
end;

procedure TMain.Memo1Exit(Sender: TObject);
begin
  Main.PopupMenu := PopupMenu1;
end;

procedure TMain.Timer1Timer(Sender: TObject);
var
  r, RT: TRect;
  W, H, j, yd1, yd2: Integer;
begin
  Timer1.Enabled := False;
 // Shape1.Visible := False;
 // Shape2.Visible := False;
 // Shape3.Visible := False;
 // Shape4.Visible := False;
  W := Trunc(bmp.Width  * PicSize);
  H := Trunc(bmp.Height * PicSize);
  r := Rect(0, 0, W, H);
  if W > ScrollBox.Width then
    Panel1.Width := W
  else
    Panel1.Width := ScrollBox.Width - 5;
  if H > ScrollBox.Height then
    Panel1.Height := H
  else
    Panel1.Height := ScrollBox.Height - 5;
  Image.Picture.Bitmap.Width  := W;
  Image.Picture.Bitmap.Height := H;
  case Mode of
  1:begin
      Image.Canvas.StretchDraw(r, bmp);
      for j := 1 to i do
      begin
        yd1 := Trunc(Ysc[j] * H - 2);
        yd2 := yd1 + 6;
        Image.Canvas.Ellipse(Trunc(Xsc[j] * H) - 2, yd1, Trunc(Xsc[j] * H) + 4, yd2);
      end;
      Shape1.Left := Trunc(xs[1] * PicSize) + 1;
      Shape1.Top  := Trunc( - ys[1] * PicSize) + 1;
      Shape2.Left := Trunc(xs[2] * PicSize) + 1;
      Shape2.Top  := Trunc( - ys[2] * PicSize) + 1;
      Shape3.Left := Trunc(xs[3] * PicSize) + 1;
      Shape3.Top  := Trunc( - ys[3] * PicSize) + 1;
      Shape4.Left := Trunc(xs[4] * PicSize) + 1;
      Shape4.Top  := Trunc( - ys[4] * PicSize) + 1;
    end;
  2:begin
      j := 1;
      with Image.Canvas do
      begin
        Brush.Color := clNone;
        RT.Top := 1;
        RT.Bottom := Image.Height - 1;
        RT.Left := 1;
        RT.Right := Image.Width - 1;
        FillRect(RT);
        {$ifdef Linux}
        StretchDraw(r, bmp);
        {$else}
        Image.Canvas.StretchDraw(r, bmp);
        {$endif}
        Pen.Color := clBlue;
        repeat
          Moveto(Trunc(Xsc[j]   * H), Trunc(Ysc[j]   * H));
          Lineto(Trunc(Xsc[j+1] * H), Trunc(Ysc[j+1] * H));
          if mmCumulative.Checked then
            j := j + 1
          else
            j := j + 2;
        until j >= i;
      end;
    end;
  3: Image.Canvas.StretchDraw(r, bmp);
  end;
end;

procedure TMain.tbZoomInClick(Sender: TObject);
begin
  ChangeZoom(PicSize * Sqrt(2));
end;

procedure TMain.tbZoomOutClick(Sender: TObject);
begin
  ChangeZoom(PicSize / Sqrt(2));
end;

procedure TMain.ChangeZoom(Size: Double);
begin
  PicSize := Size;
  Timer1.Enabled := False;
  Timer1.Enabled := True;
  ScrollBox.HorzScrollBar.Position := 1;
  ScrollBox.VertScrollBar.Position := 1;
end;

procedure TMain.chkCSVformatClick(Sender: TObject);
begin
  if chkCSVformat.Checked then
    SaveAsCSV.Checked := True
  else
    SaveAsCSV.Checked := False;
  end;

procedure TMain.SaveAsCSVClick(Sender: TObject);
begin
  SaveAsCSV.Checked := not SaveAsCSV.Checked;
  chkCSVformat.Checked := SaveAsCSV.Checked;
end;

procedure TMain.mmX_logClick(Sender: TObject);
begin
  if mmX_log.Checked = False then
  begin
    mmX_log.Checked := True;
    if (Order = 0) and (x0 > 0) and (x1 > 0) then
    begin
      x0 := Log10(x0);
      x1 := Log10(x1);
      AxisConfiguration;
      mmX_log.Enabled := False;
    end;
  end
  else
    mmX_log.Checked := False;
end;

procedure TMain.mmY_logClick(Sender: TObject);
begin
  if mmY_log.Checked = False then
  begin
    mmY_log.Checked := True;
    if (Order = 0) and (y0 > 0) and (y1 > 0) then
    begin
      y0 := Log10(y0);
      y1 := Log10(y1);
      AxisConfiguration;
      mmY_log.Enabled := False;
    end;
  end
  else
    mmY_log.Checked := False;
end;

procedure TMain.PickingUpClick(Sender: TObject);
begin
  PickingUp.Checked := True;
  ImageAnalysis.Checked := False;
  MeasureingLength.Checked := False;
  ChangeMode;
  SpinEdit.Value := 3;
  NumColEdit.Value := 1;
  Mode := 1;
  Caption := 'SimpleDigitizer - ' + Msg[Mode + 16];
end;

procedure TMain.MeasureingLengthClick(Sender: TObject);
begin
  MeasureingLength.Checked := True;
  PickingUp.Checked := False;
  ImageAnalysis.Checked := False;
  ChangeMode;
  SpinEdit.Value := 3;
  NumColEdit.Value := 1;
  Mode := 2;
  Caption := 'SimpleDigitizer - ' + Msg[Mode + 16];
end;

procedure TMain.ImageAnalysisClick(Sender: TObject);
begin
  ImageAnalysis.Checked := True;
  MeasureingLength.Checked := False;
  PickingUp.Checked := False;
  ChangeMode;
  SpinEdit.Value := 1;
  Mode := 3;
  NumColEdit.Value := 3;
  Caption := 'SimpleDigitizer - ' + Msg[Mode + 16];
end;

procedure TMain.ChangeMode;
begin
  mmX_log.Enabled   := PickingUp.Checked;
  mmY_log.Enabled   := PickingUp.Checked;
  PickingUp.Checked := PickingUp.Checked;
  Undo.Visible      := PickingUp.Checked;
  tbUndo.Visible    := PickingUp.Checked;
  pmUndo.Visible    := PickingUp.Checked;
  NumColEdit.Enabled := not PickingUp.Checked;
  Panel6.Font.Color := clBlack;
  tbUndo.Visible    := PickingUp.Checked;
  pmUndo.Enabled    := PickingUp.Checked;
  Undo.Enabled      := PickingUp.Checked;
  ScanDigitize.Enabled := PickingUp.Checked;
  GetColorStatistic.Enabled := ImageAnalysis.Checked;
  GetNumPixel.Enabled       := ImageAnalysis.Checked;
  SpinEdit.Enabled   := not ImageAnalysis.Checked;
  chkCSVformat.Enabled  := PickingUp.Checked;
  NumColEdit.Enabled := not ImageAnalysis.Checked;
  mmCumulative.Enabled   := MeasureingLength.Checked;
  mmArea.Enabled         := MeasureingLength.Checked;
  GetTotalLength.Enabled := MeasureingLength.Checked;
end;

procedure TMain.ShowSimpleMessage(S: String);
begin
  //ShowMessage(S);
  //SimpleMessage.Close;
  //Application.ProcessMessages;
  SimpleMessage.Memo1.Text := S;
  SimpleMessage.Memo1.Height := 40;
  SimpleMessage.Memo1.Color := clBtnface;
  SimpleMessage.Memo1.Enabled := False;
  SimpleMessage.Height := SimpleMessage.Memo1.Top + SimpleMessage.Memo1.Height + 30;
  SimpleMessage.Top := Screen.Height - SimpleMessage.Height - Trunc(Screen.Height / 3);
  SimpleMessage.Left := Trunc((Screen.Width - SimpleMessage.Width) / 2);
  SimpleMessage.Show;
end;

procedure TMain.FormResize(Sender: TObject);
begin
  HorzScrollBar.Position := 0;
  VertScrollBar.Position := 0;
end;

procedure TMain.Timer2Timer(Sender: TObject);
begin
  Timer2.Enabled := False;
  FormResize(Sender);
end;

function TMain.Brightness(Color: Integer): Double;
var S: String;
begin
  S := IntToHex(Color, 6);
  Result := StrToInt('$' + Copy(S, 1, 2)) / 255 / 3;
  Result := StrToInt('$' + Copy(S, 3, 2)) / 255 / 3 + Result;
  Result := StrToInt('$' + Copy(S, 5, 2)) / 255 / 3 + Result;
end;

procedure TMain.ScanDigitizeClick(Sender: TObject);
var
  n, y, xn, yn: Integer;
  x, dx, UpperLimit: Double;
  x_, y_: Double;

  function CorrectedY(y: Double): Integer;
  begin
    Result := Trunc(Image.Picture.Bitmap.Height - y * PicSize);
  end;

begin
  SigFig := SpinEdit.Value;
  x  := PicSize * xs[1];
  n  := MyMessageDlg.ReceiveInteger('SimpleDigitizer', Msg[20], ' ', 30, 40, 8);
  dx := PicSize * (xs[2] - xs[1]) / n;
  dx := Max(0.5, dx);
  UpperLimit := PicSize * xs[2];
  repeat
    x := x + dx;
    for y := CorrectedY(ys[3]) to CorrectedY(ys[4]) do
    begin
      xn := Trunc(x / PicSize);
      yn := Trunc((Image.Picture.Bitmap.Height - y) / PicSize);
      x_ := Get_X_Value(xn, yn);
      y_ := Get_Y_Value(xn, yn);
      if (Brightness(Image.Picture.Bitmap.Canvas.Pixels[Trunc(x),y]) < 0.3)
        and (x_ > x0 + (x1 - x0) / 100)
        and (x_ < x1 - (x1 - x0) / 100)
        and (y_ > y0 + (y1 - y0) / 100)
        and (y_ < y1 - (y1 - y0) / 100) then
      begin
        ImageMouseUp(Sender, mbLeft, [], Trunc(x), y);
        Application.ProcessMessages;
        Break;
      end;
    end;
  until x > UpperLimit;
end;

procedure TMain.ToolButton1Click(Sender: TObject);
begin
end;

procedure TMain.Z025Click(Sender: TObject);
begin
  ChangeZoom(0.25);
end;

procedure TMain.Z050Click(Sender: TObject);
begin
  ChangeZoom(0.5);
end;

procedure TMain.Z100Click(Sender: TObject);
begin
  ChangeZoom(1.0);
end;

procedure TMain.Z200Click(Sender: TObject);
begin
  ChangeZoom(2.0);
end;

procedure TMain.Z400Click(Sender: TObject);
begin
  ChangeZoom(4.0);
end;

procedure TMain.GetColorStatisticClick(Sender: TObject);
var
  i, n, x, y, xi, yi: Integer;
begin
  if bmp.Empty then
  begin
    ShowMessage('Please load a picture');
    Exit;
  end;
  n  := MyMessageDlg.ReceiveInteger('SimpleDigitizer', Msg[21], ' ', 100, 40, 9);
  x  := Image.Picture.Bitmap.Width;
  y  := Image.Picture.Bitmap.Height;
  Memo1.Lines.BeginUpdate;
  for i := 1 to n do
  begin
    xi := Trunc(x * Random);
    yi := Trunc(y * Random);
    GetColorInformation(xi, yi);
  end;
  Memo1.Lines.EndUpdate;
end;

procedure TMain.GetColorInformation(x, y: Integer);
var
  R, G, B: Integer;
begin
 // k := ColorToRGB(bmp.Canvas.Pixels[x, y]);
  R := 0; G := 0; B := 0;
  DecodeColor(Image.Picture.Bitmap.Canvas.Pixels[x, y], R, G, B);
  Image.Canvas.Ellipse(x - 2, y - 2, x + 2, y + 2);
  Memo1.Lines.Add(IntToStr(R) + ', ' + IntToStr(G) + ', ' + IntToStr(B));
end;

procedure TMain.DecodeColor(Col: TColor; var R, G, B: Integer);
var
  i: Integer;
  S: String;
begin
  i := Col;//ColorToRGB(Col);
  {
  B := i div 65536;
  i := i - B * 65536;
  G := i div 256;
  R := i - G * 256;
  }
  S := IntToHex(i, 6);
  B := StrToInt('$' + Copy(S, 1, 2));
  G := StrToInt('$' + Copy(S, 3, 2));
  R := StrToInt('$' + Copy(S, 5, 2));
end;

function TMain.WriteSign(x: Double): String;
begin
  if x >= 0 then
    Result := ' + ' + FloatToStr(x)
  else
    Result := FloatToStr(x);
end;

procedure TMain.GetNumPixelClick(Sender: TObject);
var
  i, n, j, x, y, LastPosition: Integer;
begin
  if bmp.Empty then
  begin
    ShowMessage('Please load a picture');
    Exit;
  end;
  if ConditionDialog.ShowModal = mrOK then
  begin
    j := 0;
    ProgressBar.Visible := True;
    LastPosition := 0;
    n := bmp.Width;
    {$ifdef MSWINDOWS}
    bmp.Canvas.Pen.Color:= AlterColor;
    bmp.Canvas.Pen.Mode := pmXor;// Copy;
    {$endif}
    for x := 1 to n - 1 do
    begin
      for y := 1 to bmp.Height - 1 do
      begin
        if IsMeetTheCondition(x, y) then
          j := j + 1;
      end;
      i := Trunc(x * 100 / n);
      if i > LastPosition then
      begin
        LastPosition := i;
        ProgressBar.Position := i;
        Application.ProcessMessages;
      end;
    end;
    ProgressBar.Visible := False;
    with Memo1.Lines do
    begin
      Clear;
      Add('Number of pixels');
      Add('that match the conditions is');
      Add(IntToStr(j) + '.');
      Add('They occupy ' + FloatToStr(RoundToDecimalSF(j * 100/ bmp.Height / bmp.Width, 4)) + '%.');
      ReportConditions;
    end;
    Image.Picture.Bitmap.Assign(bmp);
    tbSave.Enabled := True;
    Save.Enabled := True;
    if ChangeColor then
      SaveDialog.FilterIndex := 2;
    SysUtils.Beep;
  end;
end;

function TMain.IsMeetTheCondition(x, y: Integer): Boolean;
var
  R, G, B: Integer;
begin
  Result := False;
  R := 0; G := 0; B := 0;
  if not BasedOnBrightness then
  begin
    DecodeColor(bmp.Canvas.Pixels[x,y], R, G, B);
    if (RedMin <= R) and (R <= RedMax) then
      if (B >= BRa1 * R + BRb1) and (B <= BRa2 * R + BRb2) then
        if (G >= GRa1 * R + GRb1) and (G <= GRa2 * R + GRb2) then
          Result := True;
  end
  else
    if Brightness(Image.Picture.Bitmap.Canvas.Pixels[x,y]) < BrightnessLimit then
      Result := True;
  if ChangeColor and Result = True then
    {$ifdef Linux}
    bmp.Canvas.Pixels[x,y] := AlterColor;
    {$else}
    bmp.Canvas.Line(x, y, x+1, y);
    {$endif}
end;


procedure TMain.mmCumulativeClick(Sender: TObject);
begin
  mmCumulative.Checked := not mmCumulative.Checked;
  if mmCumulative.Checked then
    mmArea.Checked := False;
end;

procedure TMain.mmAreaClick(Sender: TObject);
begin
  mmArea.Checked := not mmArea.Checked;
  if mmArea.Checked then
    mmCumulative.Checked := False;
end;

procedure TMain.pmClearClick(Sender: TObject);
//var j: Integer;
begin
  if mmArea.Checked then
  begin
    //Image.Canvas.Polygon(p, False, 0, i);
    {
    Map.NumNode := i;
    Map.GenerateMesh;
    Map.Left:= 0;
    Map.Top:= 0;
    Memo1.Lines.Add(FloatToStr(Map.DrawElement(clRed) * Sqr(kx)));
    }
    Vertex[1].X := 0;
    Vertex[2].X := 0;
    Vertex[3].X := 0;
    i := 0;
  end;
  V := 0;
  xs[3] := 0;
end;

procedure TMain.PopupMenu1Popup(Sender: TObject);
begin
  pmClear.Visible := mmCumulative.Visible;
  if Mode = 1 then
    pmUndo.Enabled := True
  else
    pmUndo.Enabled := False;
end;

procedure TMain.GetTotalLengthClick(Sender: TObject);
var
  i, j, k, n, x, y, yd1, yd2: Integer;
  cx, cy: array[0..100000] of Double;
  Dark: Boolean;
  Interval: Double;

  procedure Examine;
  begin
    if IsMeetTheCondition(x,y) then
    begin
      if Dark = False then
      begin
        j := j + 1;
        cx[j] := x;
        cy[j] := y;
      end;
      Dark := True;
    end
    else
      Dark := False;
  end;
begin
  if bmp.Empty then
  begin
    ShowMessage('Please load a picture');
    Exit;
  end;
  Interval := MyMessageDlg.ReceiveFloat('SimpleDigitizer', Msg[23], ' ', 1, 40, 10);
  if ConditionDialog.ShowModal = mrOK then
  begin
    j := 0;
    ProgressBar.Visible := True;
    n := bmp.Width;
    x := Trunc(0.5 * Interval / Kx);
    ProgressBar.Max := Trunc((bmp.Width + bmp.Height) * Kx / Interval);
    repeat
      Dark := False;
      y := 1;
      repeat
        Examine;
       // if Dark then
       //   y := y + Trunc(Interval * 0.3 / Kx)
       // else
        y := y + 1;
      until y >= bmp.Height - 1;
      x := x + Trunc(Interval / Kx);
      ProgressBar.StepIt;
    until x >= n - 1;
    y := Trunc(0.5 * Interval / Kx);
    k := j;
    n := bmp.Height;
    repeat
      Dark := False;
      x := 1;
      repeat
        Examine;
        //if Dark then
        //  x := x + Trunc(Interval * 0.3 / Kx)
        //else
        x := x + 1;
      until x >= bmp.Width - 1;
      y := y + Trunc(Interval / Kx);
      ProgressBar.StepIt;
    until y >= n - 1;
    {$ifdef Linux}
    Image.Canvas.Pen.Color:= AlterColor;
    Image.Canvas.Pen.Mode := pmNotXor;
    {$else}
    Image.Canvas.Pen.Color:= GetReverseColor(AlterColor);
    Image.Canvas.Pen.Mode := pmXor;// Copy;
    {$endif}
    x := Trunc(0.5 * Interval / Kx);
    repeat
      Image.Canvas.MoveTo(x, 1);
      Image.Canvas.LineTo(x, bmp.Height);
      x := x + Trunc(Interval / Kx);
    until x > bmp.Width;
    y := Trunc(0.5 * Interval / Kx);
    repeat
      Image.Canvas.MoveTo(1, y);
      Image.Canvas.LineTo(bmp.Width, y);
      y := y + Trunc(Interval / Kx);
    until y > bmp.Height;
    Image.Canvas.Brush.Style := bsClear;
    for i := 1 to j do
    begin
      yd1 := Trunc(PicSize * cy[i]) - 2;
      yd2 := yd1 + 6;
      Image.Canvas.Ellipse(Trunc(PicSize * cx[i]) - 2, yd1, Trunc(PicSize * cx[i]) + 4, yd2);
    end;
    j := j - k;
    ProgressBar.Visible := False;
    Memo1.Lines.Add('Total length = ' + FloatToStr(0.25 * (j + k) * 3.14 * Interval));
    ReportConditions;
  end;
end;

function TMain.GetReverseColor(Color: Integer): Integer;
var dumstr: String;
begin
  dumstr := IntToHex(Color, 6);
  Result := (255 - StrToInt('$' + Copy(dumstr, 1, 2))) * 256 * 256;
  Result := (255 - StrToInt('$' + Copy(dumstr, 3, 2))) * 256 + Result;
  Result := (255 - StrToInt('$' + Copy(dumstr, 5, 2)))       + Result;
end;

procedure TMain.ReportConditions;
begin
  with Memo1.Lines do
  begin
    Add('The conditions are :');
    if not BasedOnBrightness then
    begin
      if RedMin > 0 then
        Add('Red > ' + IntToStr(RedMin));
      if RedMax < 255 then
        Add('Red < ' + IntToStr(RedMax));
      if ConditionDialog.chkBlueToRedMin.Checked then
        Add('Blue > ' + FloatToStr(BRa1) + 'Red' + WriteSign(BRb1));
      if ConditionDialog.chkBlueToRedMax.Checked then
        Add('Blue < ' + FloatToStr(BRa2) + 'Red' + WriteSign(BRb2));
      if ConditionDialog.chkGreenToRedMin.Checked then
        Add('Green > ' + FloatToStr(GRa1) + 'Red' + WriteSign(GRb1));
      if ConditionDialog.chkGreenToRedMax.Checked then
        Add('Green < ' + FloatToStr(GRa2) + 'Red' + WriteSign(GRb2));
    end
    else
       Add('Brightness < ' + FloatToStr(BrightnessLimit));
  end;
end;

Initialization
{$I Digitizer_Main.lrs}
end.
